import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:listenable/listenable.dart';

part 'main.g.dart';

void main() {
  runApp(MaterialApp(
    home: MyApp(),
  ));
}

enum Gender { MALE, FEMALE }

class Person = _Person with _Person$Listenable;

@ListenableModel()
abstract class _Person {

  String name;
  String cellPhone;
  @ListenableProperty()
  double age = 0;
  Gender gender;
  @ListenableProperty()
  bool favorite = false;

  @override
  String toString() {
    return '_Person{name: $name, cellPhone: $cellPhone, age: $age, gender: $gender, favorite: $favorite}';
  }

}

class DirectoryController = _DirectoryController
    with _DirectoryController$Listenable;

@ListenableModel()
abstract class _DirectoryController {
  @ListenableProperty()
  double age = 0;

  @ListenableProperty()
  List<Person> contacts = List<Person>();

  @ListenableProperty()
  Person currentPerson = Person();

  void newContact() {
    currentPerson = Person();
  }

  void addContact() {
    contacts.add(currentPerson);
    currentPerson = Person();

    contacts = List.of(contacts);
  }

  void deleteContact(Person person) {
    contacts.remove(person);
    contacts = List.of(contacts);
  }

  _DirectoryController() {
    print('Creando _DirectoryController');
  }
}

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        title: 'Flutter Demo',
        theme: ThemeData(
          primarySwatch: Colors.blue,
        ),
        home: Scaffold(
          appBar: AppBar(
            title: Text("Hola"),
          ),
          body: Center(
            child: SafeArea(
              child: DirectoryWidget(),
            ),
          ),
        ));
  }
}

class DirectoryWidget extends StatefulWidget {
  @override
  _DirectoryWidgetState createState() => _DirectoryWidgetState();
}

class _DirectoryWidgetState extends State<DirectoryWidget> {
  final controller = DirectoryController();

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        DirectoryForm(controller),
        Expanded(
          child: ContactListWidget(controller),
        ),
      ],
    );
  }
}

class ContactListWidget extends StatelessWidget {
  final DirectoryController controller;

  ContactListWidget(this.controller);

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: 200,
      child: ValueListenableBuilder<List<Person>>(
        valueListenable: controller.contactsListenable,
          builder: (context, snapshot, _) {
            print('${controller.contacts.length}');
            return ListView.builder(
              itemCount: controller.contacts.length,
              itemBuilder: (BuildContext context, int index) {
                final contact = snapshot[index];
                return Dismissible(
                  key: ValueKey(contact),
                  onDismissed: (direction) {
                    controller.deleteContact(snapshot[index]);
                  },
                  child: ContactInformation(controller, contact),
                );
              },
            );
          }),
    );
  }
}

class ContactInformation extends StatelessWidget {
  final DirectoryController controller;
  final Person contact;

  ContactInformation(this.controller, this.contact);

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        controller.currentPerson = contact;
      },
      child: ValueListenableBuilder<Person>(
          valueListenable: contact.listenable,
          builder: (context, person, _) {
            return ListTile(
              key: ValueKey(contact),
              title: Text(contact.name),
              subtitle: Text(contact.cellPhone),
              leading: IconButton(
                icon: Icon(
                    person.favorite ? Icons.star : Icons.star_border),
                tooltip: 'Increase volume by 10',
                onPressed: () {
                  person.favorite = !person.favorite;
                },
              ),
              trailing: ValueListenableBuilder<double>(
                  valueListenable: contact.ageListenable,
                  builder: (context, age, _) {
                    return Text('${age}');
                  }),
            );
          }),
    );
  }
}

class DirectoryForm extends StatelessWidget {
  final DirectoryController controller;
  final FocusNode _focusNode = FocusNode();

  DirectoryForm(this.controller);

  @override
  Widget build(BuildContext context) {
    return ValueListenableBuilder<Person>(
        valueListenable: controller.currentPersonListenable,
        builder: (context, person, _) {
          print('Rebuilding');
          try {
            return Column(
              children: <Widget>[
                TextField(
                  focusNode: _focusNode,
                  controller: TextEditingController(text: person.name),
                //  showCursor : false,
                  decoration: InputDecoration(
                    border: OutlineInputBorder(),
                    labelText: 'Name',
                  ),
                  onChanged: (nValue) {
                    person.name = nValue;
                  },
                ),
                Container(
                  height: 12,
                ),
                TextField(
               //   showCursor : false,
                  controller:
                      TextEditingController(text: person.cellPhone),
                  decoration: InputDecoration(
                    border: OutlineInputBorder(),
                    labelText: 'CellPhone',
                  ),
                  keyboardType: TextInputType.number,
                  onChanged: (nValue) {
                    person.cellPhone = nValue;
                  },
                ),
                ValueListenableBuilder<double>(
                  valueListenable: person.ageListenable,
                  builder: (context, ageSnapshot, _) {
                    return new AgeSlider(person, ageSnapshot);
                  },
                ),
                Container(
                  height: 1.5,
                  width: MediaQuery.of(context).size.width - 12,
                  color: Colors.black26,
                ),
                RaisedButton(
                  child: Text('New Contact'),
                  onPressed: () {
                    controller.newContact();
                  },
                ),
                RaisedButton(
                  child: Text('Add'),
                  onPressed: () {
                    controller.addContact();
                  },
                )
              ],
            );
          } finally {
            FocusScope.of(context).requestFocus(_focusNode);
          }
        });
  }
}

class AgeSlider extends StatelessWidget {
  final Person person;
  final double currentValue;

  const AgeSlider(this.person, this.currentValue, {Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Slider(
      value: currentValue,
      min: 0,
      max: 100,
      divisions: 100,
      label: 'Age ${currentValue}',
      onChanged: (nValue) {
        person.age = nValue.round().toDouble();
      },
    );
  }
}
